package com.aitrainer.api.test

import com.aitrainer.api.controller.PackageController
import com.aitrainer.api.model.*
import com.aitrainer.api.repository.*
import org.junit.jupiter.api.Test
import org.junit.jupiter.api.TestInstance
import org.springframework.beans.factory.annotation.Autowired
import org.springframework.boot.test.context.SpringBootTest
import org.springframework.http.ResponseEntity
import com.google.gson.Gson
import com.google.gson.reflect.TypeToken
import kotlin.test.assertEquals
import kotlin.test.assertTrue

@SpringBootTest
@TestInstance(TestInstance.Lifecycle.PER_CLASS)
class AppPackageTest {

    @Autowired
    private lateinit var exerciseAbilityRepository: ExerciseAbilityRepository
    @Autowired
    private lateinit var exerciseTypeRepository: ExerciseTypeRepository
    @Autowired
    private lateinit var  exerciseTreeRepository: ExerciseTreeRepository
    @Autowired
    private lateinit var  propertyRepository: PropertyRepository
    @Autowired
    private lateinit var  productRepository: ProductRepository
    @Autowired
    private lateinit var  exerciseDeviceRepository: ExerciseDeviceRepository
    @Autowired
    private lateinit var exerciseTreeParentsRepository: ExerciseTreeParentsRepository

    @Test
    fun testAppPackage() {
        val gson = Gson()

        val controller = PackageController(
            exerciseAbilityRepository,
            exerciseTypeRepository,
            exerciseTreeRepository,
            propertyRepository,
            productRepository,
            exerciseDeviceRepository,
            exerciseTreeParentsRepository
        )
        val response: ResponseEntity<*> = controller.getPackageData()

        val appPackageJson: String = response.body as String
        assertTrue(appPackageJson.isNotEmpty())


        val packages = appPackageJson.split("|||").toTypedArray()

        packages.forEach {
            val record = it.split("***")
            print(record[0] + "\n")
            if ( record[0] == ExerciseType::class.simpleName) {
                print("List ExerciseType: " + record[1])
                val exerciseTypeJson: String = record[1]
                val type = object : TypeToken<List<ExerciseType?>?>() {}.type
                val listExerciseType: List<ExerciseType> = gson.fromJson(exerciseTypeJson, type)
                assertTrue(listExerciseType.isNotEmpty())
                assertEquals(listExerciseType[1].name, "Biceps")
                assertEquals(listExerciseType[1].translations[0].name, "Bicepsz")
            } else if (record[0] == ExerciseTree::class.simpleName) {
                val exerciseTreeJson: String = record[1]
                val type = object : TypeToken<List<ExerciseTree?>?>() {}.type
                val listExerciseTree: List<ExerciseTree> = gson.fromJson(exerciseTreeJson, type)
                assertTrue(listExerciseTree.isNotEmpty())
                assertEquals(listExerciseTree[1].name, "Strength")
                assertEquals(listExerciseTree[1].translations[0].name, "Erő!")
            } else if (record[0] == Property::class.simpleName) {
                val propertyJson: String = record[1]
                val type = object : TypeToken<List<Property?>?>() {}.type
                val listProperty: List<Property> = gson.fromJson(propertyJson, type)
                assertTrue(listProperty.isNotEmpty())
                assertEquals(listProperty[2].propertyName, "Chest")
                assertEquals(listProperty[0].translations[0].propertyName, "Tömeg")
            } else if (record[0] == ExerciseTreeParents::class.simpleName) {
                val exerciseTreeParentsJson: String = record[1]
                val type = object : TypeToken<List<ExerciseTreeParents?>?>() {}.type
                val listParents: List<ExerciseTreeParents> = gson.fromJson(exerciseTreeParentsJson, type)
                assertTrue(listParents.isNotEmpty())
                assertEquals(listParents[2].exerciseTreeParentId, 0)
                assertEquals(listParents[6].exerciseTreeParentId, 4)
                assertEquals(listParents[6].exerciseTreeChildId, 9)
            }
        }



    }
}